export declare class HilbertCurveBase {
    private _x;
    private _y;
    private _d;
    private _width;
    private _height;
    private _callback;
    private _level;
    walk(width: number, height: number, visitorCallback: (x: number, y: number, d: number) => void): void;
    private _walkHilbert(direction);
    private _visit(direction);
}
