/**
 * @preserve
 * Copyright 2015-2016 Igor Bezkrovnyi
 * All rights reserved. (MIT Licensed)
 *
 * ssim.ts - part of Image Quantization Library
 */
import { PointContainer } from "../utils/pointContainer";
export declare class SSIM {
    compare(image1: PointContainer, image2: PointContainer): number;
    private _iterate(image1, image2, callback);
    private _calculateLumaValuesForWindow(image, x, y, width, height);
    private _calculateAverageLuma(lumaValues);
}
